import { logger } from '@/module/logger.ts';
import { program, Command, assistantConfig } from '@/program.ts';
import { ProviderManager, ProviderMap } from '@kevisual/ai-center';
const runAiCommand = new Command('ai')
  .description('AI 对话助手')
  .option('-t, --text <text>', '应用类型', '')
  .action(async (options) => {
    assistantConfig.checkMounted();
    const config = assistantConfig.getCacheAssistantConfig();
    const ai = config.ai || {};
    if (!ai.enabled) {
      logger.error('AI 功能未启用，请在助手配置文件中启用 AI 功能');
      return;
    }
    if (!options.text) {
      logger.error('请输入要发送给 AI 的文本');
      return;
    }
    if (!ProviderMap[ai.provider]) {
      logger.error('不支持的 AI 提供商，请检查配置文件中的提供商名称');
      const supportedProviders = Object.keys(ProviderMap).join(', ');
      logger.error(`支持的提供商有：${supportedProviders}`);
      return;
    }
    const provider = new ProviderManager({
      provider: ai.provider,
      apiKey: ai.apiKey,
      model: ai.model,
    });
    const result = await provider.chat([
      {
        role: 'user',
        content: options.text,
      },
    ]);
    logger.info(result.choices[0].message.content);
  });

program.addCommand(runAiCommand);
