import { Manager } from '@kevisual/local-app-manager/manager';
import type { AssistantConfig } from '@/module/assistant/index.ts';
import { parseIfJson } from '@/module/assistant/index.ts';
import path from 'node:path';
import fs from 'node:fs';
import glob from 'fast-glob';
import type { App } from '@kevisual/router';
export class AssistantApp extends Manager {
  config: AssistantConfig;
  pagesPath: string;
  constructor(config: AssistantConfig, mainApp?: App) {
    config.checkMounted();
    const appsPath = config?.configPath?.appsDir || path.join(process.cwd(), 'apps');
    const pagesPath = config?.configPath?.pagesDir || path.join(process.cwd(), 'pages');
    const appsConfigPath = config.configPath?.appsConfigPath;
    const configFimename = path.basename(appsConfigPath || '');
    super({
      appsPath,
      configFilename: configFimename,
      mainApp: mainApp,
    });
    this.pagesPath = pagesPath;
    this.config = config;
  }
  async pageList() {
    const pages = await glob(['*/*/package.json'], {
      cwd: this.pagesPath,
      onlyFiles: true,
    });
    const pagesParse = pages.map((page) => {
      const [user, app] = page.split('/');
      const contentStr = fs.readFileSync(path.join(this.pagesPath, page), 'utf-8');
      const content = parseIfJson(contentStr);
      return {
        user,
        app,
        version: content?.version,
        title: content?.title || '',
        description: content?.description || '',
        content,
      };
    });
    return pagesParse;
  }
  async getPageAndAppList() {
    const root = this.config.configPath.configDir;
    const pages = await glob([root + '/apps/*/package.json', root + '/pages/*/*/package.json'], {
      cwd: root,
      onlyFiles: true,
    });
    const pagesParse = pages.map((page) => {
      const relativePath = path.relative(root, page);
      const contentStr = fs.readFileSync(path.join(page), 'utf-8');
      const content = parseIfJson(contentStr);
      if (!content.appType) {
        const isWeb = relativePath.startsWith('pages/');
        content.appType = isWeb ? 'web' : 'app';
      }
      return {
        ...content,
        filepath: relativePath,
      };
    });
    return pagesParse;
  }
}
