import path from 'node:path';
import fs from 'node:fs';
import { checkFileDir, checkFileExists } from '../file/index.js';
type SimpleObject = {
  [key: string]: any;
};
type StorageOptions = {
  dir?: string;
};
export class StorageCore {
  storagePath: string;
  constructor(opts?: StorageOptions) {
    this.storagePath = opts?.dir || process.cwd();
  }
  async getData<T = SimpleObject>(key: string = 'data.json', fileDirectory?: string): Promise<T | null> {
    const storageFilePath = path.join(this.storagePath, fileDirectory, key);
    if (!checkFileExists(storageFilePath)) {
      return null;
    }
    try {
      const data = fs.readFileSync(storageFilePath, 'utf-8');
      return JSON.parse(data) as T;
    } catch (error) {
      console.error(`Error reading data from ${storageFilePath}:`, error);
      return null;
    }
  }
  async setData<T = SimpleObject>(data: T, key: string = 'data.json', fileDirectory?: string): Promise<void> {
    const storageFilePath = path.join(this.storagePath, fileDirectory, key);
    try {
      checkFileDir(storageFilePath, true);
      fs.writeFileSync(storageFilePath, JSON.stringify(data, null, 2), 'utf-8');
    } catch (error) {
      console.error(`Error writing data to ${storageFilePath}:`, error);
    }
  }
  async deleteData(key: string = 'data.json', fileDirectory?: string): Promise<void> {
    const storageFilePath = path.join(this.storagePath, fileDirectory, key);
    try {
      if (checkFileExists(storageFilePath)) {
        fs.unlinkSync(storageFilePath);
      }
    } catch (error) {
      console.error(`Error deleting data from ${storageFilePath}:`, error);
    }
  }
}
