import MD5 from 'crypto-js/md5.js';
import { getBaseURL, queryLogin } from '../query.ts';
import { chalk } from '../chalk.ts';
import jsonwebtoken from 'jsonwebtoken';
import { BaseLoad } from '@kevisual/load';

type LoginWithWebOptions = {};
export const loginWithWeb = async (opts?: LoginWithWebOptions) => {
  const baseURL = getBaseURL();

  const res = queryLogin.loginWithWeb(baseURL, { MD5, jsonwebtoken });
  console.log(chalk.blue(res.url));
  return res;
};

type PollLoginOptions = {
  tokenSecret: string;
};
export const pollLoginStatus = async (token: string, opts: PollLoginOptions) => {
  const load = new BaseLoad();
  load.load(
    async () => {
      const res = await queryLogin.checkLoginStatus(token);
      if (res.code === 500) {
        load.cancel('check-login-status');
      }
      return res;
    },
    {
      key: 'check-login-status',
      isReRun: true,
      checkSuccess: (data) => {
        return data?.code === 200;
      },
    },
  );
  const res = await load.hasLoaded('check-login-status', {
    timeout: 60 * 3 * 1000, // 5分钟超时
  });
  if (res.code === 200 && res.data?.code === 200) {
    try {
      console.log(chalk.green('网页登录成功'));
      return;
    } catch (error) {
      console.log(chalk.red('登录失败'), error);
      return;
    }
  }
  console.log(chalk.red('登录失败'), res);
};

export const loginInCommand = async () => {
  const baseURL = getBaseURL();
  const res = queryLogin.loginWithWeb(baseURL, { MD5, jsonwebtoken });
  console.log(chalk.blue(res.url));
  await pollLoginStatus(res.token, { tokenSecret: res.tokenSecret });
  return res.url;
};
